
Function AutoCorrelate, X, variance=variance, ftx=ftx
;+
;PURPOSE: calculate ACF of a time series X using FFT. Also provides the
;FFT of X.  Stolen from idl documentation of
;A_CORRELATE. 
;
;Note this subtracts the mean beforehand, as does A_CORRELATE. Note that
;this result, using FFT, differs somewhat from that using A_CORRELATE.
;
;CALLING SEQUENCE:
;acf = AUTOCORRELATE( x, variance=variance, ftx=ftx)
;
;INPUTS:
;X, the time series.
;
;OUTPUTS:
;ACF is returned. We return an array of real numbers, because the
;imaginary parts are all zero.
;FTX, the FFT of X, a complex array.
;VARIANCE, the variance of X. Multiply ACF by VARIANCE to get the
;       autocovariance. 
;
;-

ftx = FFT(X - mean(x),-1)   

acf=real_part(FFT(Ftx * CONJ(Ftx), 1))
variance=acf[0]

return, acf[0l:n_elements(x)/2l]/ variance

;Result = A_CORRELATE(X, Lag [, /COVARIANCE] [, /DOUBLE] )

END
